# -*- coding: utf-8 -*-

import datetime
import time
import re

import dateutil
import dateutil.parser
from sentry_sdk import capture_exception
from sp_api.base.exceptions import SellingApiForbiddenException, SellingApiBadRequestException

from dropshipping.spapi.exceptions import SellingApiInvalidAsinException

from em_tasks import logger
from em_tasks.shopify.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.spapi import exceptions_to_retry, exceptions_not_retry


class SpapiUpdateItemOffersTask():
  def __init__(self, spapi, offer_service, marketplace, asins, condition='new', error_service=None):
    self.spapi = spapi
    self.error_service = error_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.asins = asins
    self.condition = condition.lower()

  def run(self):
    offer_type = 'lowest_offer_listings'

    offers = None
    while True:
      try:
        offers = self.spapi.get_item_offers_batch(self.marketplace, self.asins, self.condition)
        break
      except SellingApiForbiddenException as e:
        raise e
      except exceptions_to_retry as e:
        time.sleep(3)
      except SellingApiInvalidAsinException as e:
        matched = re.match(r'([A-Z0-9]{10}) is an invalid ASIN', e.message)
        if not matched:
          break

        asin = matched.groups()[0]
        if asin in self.asins:
          self.asins.remove(asin)
      except SellingApiBadRequestException:
        break
      except exceptions_not_retry as e:
        break

    if not offers:
      if self.error_service:
        try:
          self.error_service.save_no_offer_asins(
            "offer", self.asins, self.marketplace
          )
        except:
          pass

      return

    try:
      result = self.offer_service.save_item_offers(
        offer_type, offers, self.marketplace, self.condition)
      if result:
        logger.debug('[OfferSaved] %s', offers.keys())
      else:
        logger.debug('[OfferSaveFailed] Fetched: %s', offers)
    except Exception as e:
      logger.debug('[OfferSaveFailed] Fetched: %s', offers)

      try:
        capture_exception(e)
      except:
        pass

    return offers
