# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.store.models import *


class AmzProductsFilter():
  def __init__(self, store_code, marketplace, filter_conds = None):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.blacklist_filter = BlacklistFilter()

    self.marketplace = marketplace.lower()
    self.pc = AmzProductConverter(self.marketplace)

    dft_filter_conds = {
      'sales_rank_min': 5000000,
      'weight_max': 2.5,
      'dimension_max': 20,
      'export': False
    }

    self.filter_conds = dft_filter_conds
    if filter_conds:
      self.filter_conds.update(filter_conds)

    self.products_filter_pipeline = AmzProductsFilterPipeline(
      self.marketplace, self.filter_conds['weight_max'],
      self.filter_conds['dimension_max'], self.filter_conds['sales_rank_min'],
      self.filter_conds['export'])

    self.uploaded_prods = dict()
    self.load_uploaded_products()

  def filter_products(self, products):
    result = dict()
    try:
      prods_buf = dict()
      for prod_id, prod in products.items():
        if prod_id in self.uploaded_prods:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Uploaded - %s', prod_id, self.uploaded_prods[prod_id])
          result[prod_id] = {'passed': False, 'reason': 'Uploaded'}
          continue

        r = self.products_filter_pipeline.check_product(prod)
        passed = r.get('passed', True)
        reason = r.get('reason', '')
        if not passed:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', prod_id, reason)
          result[prod_id] = {'passed': False, 'reason': '[Unuploadable] {}'.format(reason)}
          continue

        r = self.check_blacklist(prod)
        passed = r.get('passed', True)
        reason = r.get('reason', '')
        if not passed:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', prod_id, reason)
          result[prod_id] = {'passed': False, 'reason': '[Blacklisted] {}'.format(reason)}
          continue

        result[prod_id] = {'passed': True, 'reason': ''}
    finally:
      pass

    return result

  def load_uploaded_products(self):
    if self.uploaded_prods:
      return

    try:
      for prod in self.product_util.get_amz_products(self.marketplace):
        source_product_id = prod.get('source_product_id', prod.get('asin', None))
        if not source_product_id:
          continue

        self.uploaded_prods[source_product_id] = None

      logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(self.uploaded_prods))
    except Exception as e:
      logger.exception(e)

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}
    s = '{}-{}'.format(product.get('title', '').lower(), self.pc.get_brand(product).lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted
