# -*- coding: utf-8 -*-

import datetime
from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.store.models import *


class AmzProductsAnalyzerAndSend():
  def __init__(self, marketplace, product_service, offer_service, mongo_client, condition='new'):
    self.mongo_client = mongo_client
    self.product_service = product_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.condition = condition
    self.pc = AmzProductConverter(self.marketplace)

    self.blacklist_filter = BlacklistFilter()
    self.sales_rank_min = 1500000
    if self.marketplace == 'us':
      self.sales_rank_min = 500000

    self.weight_max = 2.5
    self.dimension_max = 20

    self.products_filter_pipeline = AmzProductsFilterPipeline(self.marketplace, self.weight_max, self.dimension_max, self.sales_rank_min)
    self.product_indice_name = 'amz_products_api_{}'.format(marketplace.lower())
    self.headers = ['asin', 'uploadable', 'has_product_info', 'filtered', 'filtered_reason', 'has_offers', 'has_offer', 'offer']
    self.asins_without_offer = set()

  def run(self):
    batch_size = 500
    asin_dict = dict()
    asins = set()
    one_week_agon = datetime.datetime.now() - datetime.timedelta(weeks=1)
    query = {"time": {"$gt": one_week_agon}}

    for doc in self.mongo_client.client[f"amazon_{self.marketplace}"].product_list.find(query, batch_size=40000):
        asins.add(doc["asin"])

    for asin in asins:
        asin_dict[asin.strip()] = None
        if len(asin_dict) < batch_size:
            continue
        yield self.process_products(list(asin_dict.keys()))
        asin_dict = {}

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}

    s = '{}-{}'.format(product.get('title', '').lower(), self.pc.get_brand(product).lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted

  def process_products(self, asins):
    result = {asin:{} for asin in asins}

    products_info = self.product_service.search_products(self.product_indice_name, asins)
    for asin in asins:
      product_info = products_info.get(asin, None) if products_info else None
      if product_info:
        result[asin]['has_product_info'] = True
        res = self.products_filter_pipeline.check_product(product_info)
        passed = res.get('passed', True)
        reason = res.get('reason', '')
        if passed:
            blacklist_result = self.check_blacklist(product_info)
            passed = blacklist_result.get('passed', True)
            reason = blacklist_result.get('reason', '')
            if not passed:
              result[asin]['filtered'] = True
              result[asin]['filtered_reason'] = reason
        else:
          result[asin]['filtered'] = True
          result[asin]['filtered_reason'] = reason
      else:
        result[asin]['has_product_info'] = False

    offers_result = self.offer_service.analyze_offers(self.marketplace, asins, self.condition)
    for asin, offer_result in offers_result.items():
      result[asin].update(offer_result)

    for asin, res in result.items():
      item = {
        'asin': asin,
        'has_product_info': res.get('has_product_info', False),
        'filtered': res.get('filtered', False),
        'filtered_reason': res.get('filtered_reason', None),
        'has_offers': res.get('has_offers', False),
        'has_offer': res.get('has_offer', False),
        'offer': res.get('offer', None)
      }
      item['uploadable'] = item['has_product_info'] and not item['filtered'] and not item["has_offer"]
      if item['uploadable']:
          yield asin

