# -*- coding: utf-8 -*-

import datetime
import json
import time
from typing import Dict, Optional
from multiprocessing.pool import ThreadPool

import dateparser

from em_tasks import logger
from em_tasks.spree.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil
from em_tasks.utils.spree.base_repricer import SpreeBaseRepricer
from em_tasks.utils.spree.validations import validate_prods


class SpreeYesstyleRepricer(SpreeBaseRepricer):
  def __init__(self, store_code, product_service, price_calculator, ttl=7):
    super(SpreeYesstyleRepricer, self).__init__()
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
    # self.product_util = ProductUtil('http://localhost:3000', credential['api_key'], credential.get('api_version', 'v1'))
    self.product_service = product_service
    self.price_calc = price_calculator
    self.indice_name = 'yesstyle_products'
    self.asian_beauty_indice = 'asianbeauty_products'
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl)
    self.pool = ThreadPool(3)

  def run(self):
    batch_size = 250
    products = dict()
    for yesstyle_prod in self.product_util.get_yesstyle_products():
      products[yesstyle_prod['source_product_id']] = yesstyle_prod

      if len(products) < batch_size:
        continue

      self.process_products(products)
      products.clear()

    if products:
      self.process_products(products)
      products = []

    self.pool.close()
    self.pool.join()

  def get_source_variant_id_from_sku(self, sku):
      return sku.replace("_", "-").split("-").pop()

  def get_variant_id(self, product, source_variant):
      for variant in product["variants"]:
          source_variant_id = self.get_source_variant_id_from_sku(variant["sku"])
          if str(source_variant_id) == str(source_variant["variant_id"]):
              return variant["variant_id"]

  def is_price_changed(self, variant, offer):
    spree_price = variant['price']
    offer_price = offer['price']
    return abs(spree_price - offer_price) > 1

  def fetch_asian_products(self, product_ids) -> dict:
    max_retries = 3
    while max_retries > 0:
        try:
            products = self.product_service.search_products(self.asian_beauty_indice, product_ids)
            if isinstance(products, dict):
                return products
            logger.info('[ProductService] Temporary unavailable! Retrying in 10s...')
            max_retries -= 1
            time.sleep(10)
        except Exception as e:
            logger.exception(f"Error fetching products: {e}")
            max_retries -= 1
            time.sleep(3)

    return {}

  def fetch_products(self, product_ids) -> dict:
    max_retries = 3
    while max_retries > 0:
        try:
            products = self.product_service.search_products(self.indice_name, product_ids)
            if isinstance(products, dict):
                return products
            logger.info('[ProductService] Temporary unavailable! Retrying in 10s...')
            max_retries -= 1
            time.sleep(10)
        except Exception as e:
            logger.exception(f"Error fetching products: {e}")
            max_retries -= 1
            time.sleep(3)

    return {}

  def calculate_offer(self, product: dict) -> dict:
    try:
      src_product_price = float(product.get('price', 0))
      src_shipping_fee = float(product.get('shipping_fee', 0)) or 0
    except ValueError:
      return {}

    currency = product.get('currency', 'USD')
    src_offer = {'price': src_product_price + src_shipping_fee, 'currency': currency}
    offer = self.price_calc.calc_offer(src_offer)
    if offer:
        offer.update({'price': src_offer['price'] - 1, 'quantity': 50})
        src_offer['price'] = round(float(src_offer['price']) * 0.4, 2)
        offer['src_offer'] = src_offer

    return offer

  def is_product_valid(self, product: dict) -> bool:
    update_date_s = product.get('date') or product.get('time')
    try:
        update_date = dateparser.parse(update_date_s).date() if update_date_s else None
        if update_date and update_date > self.expire_date:
            return True
    except:
        pass

    return False

  def is_product_out_of_stock(self, product: dict) -> bool:
    existence = product.get('existence', True)
    src_product_price = float(product.get('price', 0))
    if existence is False or src_product_price < 2:
      return True
    return False

  def is_valid_date(self, time_str):
    try:
      updated_at = datetime.datetime.strptime(time_str)
      return updated_at > datetime.datetime.now() - datetime.timedelta(days=7)
    except Exception as e:
      return True

  def is_product_long_edd(self, product):
    if not product:
      return False

    return '21 days' in product["availability"]

  def is_product_not_sellable(self, product, asin_beauty_products) -> bool:
    if product['product_id'] not in asin_beauty_products:
      return False

    if self.is_valid_date(asin_beauty_products[product['product_id']].get('date')):
      return not asin_beauty_products[product['product_id']]['existence']

    if self.is_product_long_edd(asin_beauty_products[product['product_id']]):
        return True

    return False

  def process_single_product(self, product: dict, spree_prod: dict, product_ids_with_valid_price: dict, asin_beauty_products):
    if self.is_product_out_of_stock(product):
        spree_prod['offer'] = {'price': 0, 'quantity': 0}
        product_ids_with_valid_price[spree_prod['product_id']] = None
        return

    if self.is_product_not_sellable(product, asin_beauty_products):
        spree_prod['offer'] = {'price': 0, 'quantity': 0}
        product_ids_with_valid_price[spree_prod['product_id']] = None
        return

    offer = self.calculate_offer(product)
    if offer:
        if self.is_price_changed(spree_prod['variants'][0], offer):
            product_ids_with_valid_price[spree_prod['product_id']] = None
            spree_prod['offer'] = offer
    else:
        spree_prod['offer'] = offer


  def process_products(self, prods: Dict[str, dict]) -> Optional[dict]:
    validate_prods(prods)

    product_ids = list(prods.keys())
    products = self.fetch_products(product_ids)
    if not products:
      return

    asin_beauty_products = self.fetch_asian_products(product_ids)
    product_ids_with_valid_price = {}

    for pid, product in products.items():
      if pid not in prods or not self.is_product_valid(product):
        continue

      self.process_single_product(product, prods[pid], product_ids_with_valid_price, asin_beauty_products)

    offers = {prod['product_id']: prod for _, prod in prods.items() if prod['product_id'] in product_ids_with_valid_price}
    return self.product_util.set_products_offer(offers, self.pool) if offers else None
